local love = require "love"
local push = require "push"

function Ball(x, y, cue)
  local ballRadius = 5
  local startX = x
  local startY = y
  local powerBar = love.graphics.newImage("assets/powerBar.png")
  local powerBarMarker = love.graphics.newImage("assets/powerBarMarker.png")
  local poolCue = love.graphics.newImage("assets/poolCue.png")

  

  return {
    x = x or 0,
    y = y or 0,
    cue = cue or false,
    body = nil,
    shape = nil,
    fixture = nil,
    power = 0,

    vecx = 0,
    vecy = 0,
    vlen = 0,
    vecAngle = 0,

    rate = 100,
    step = 0,
    dir = "up",
    second = 0,
    counter = 1,

    markerX = 162,

    createBall = function(self)
      self.body = love.physics.newBody(world, self.x, self.y, "dynamic");
      self.shape = love.physics.newCircleShape(ballRadius);
      self.fixture = love.physics.newFixture(self.body, self.shape)
      self.fixture:setRestitution(1)
      self.fixture:setFriction(0.5)

      if cue then
        self.fixture:setUserData("Cue Ball")
      else
        self.fixture:setUserData("Ball")
      end
    end,

    draw = function (self)
      local _x, _y = self.body:getPosition()
      if cue then
        local cueBallImage = love.graphics.newImage("assets/cue_ball.png")
        love.graphics.draw(cueBallImage, _x, _y)
      else
        local ballImage = love.graphics.newImage("assets/ball.png")
        love.graphics.draw(ballImage, _x, _y)
      end
    end,

    drawPowerBar = function (self)
      local startX = 92
      self.markerX = (self.power * 0.77) + startX+2

      if self.step == 1 then
        love.graphics.push()
        love.graphics.scale(1.5)
        love.graphics.draw(powerBar, startX, 2)
        love.graphics.draw(powerBarMarker,self.markerX, 4)
        love.graphics.pop()
      end
    end,

    drawPoolCue = function (self)
      
    end,

    resetBall = function (self)
      if love.keyboard.isDown("f") then
        
        self.body:setLinearVelocity(0,0)
        self.body:setPosition(startX, startY)
      end
    end,

    moveBall = function (self)
      local vx, vy = self.body:getLinearVelocity()
      
      local norm = math.sqrt(vx*vx + vy*vy)
      if norm > 0.0001 then
          if ((vx or vy) > 300) or ((vx or vy) < -300) then
            self.body:applyForce(vx/norm*-120, vy/norm*-120)
          elseif ((vx or vy) > 200) or ((vx or vy) < -200) then
              self.body:applyForce(vx/norm*-105, vy/norm*-105)
          elseif ((vx or vy) > 150) or ((vx or vy) < -150) then
              self.body:applyForce(vx/norm*-95, vy/norm*-95)
          elseif ((vx or vy) > 90) or ((vx or vy) < -90) then
              self.body:applyForce(vx/norm*-70, vy/norm*-70)
          elseif ((vx or vy) > 50) or ((vx or vy) < -50) then
              self.body:applyForce(vx/norm*-70, vy/norm*-70)
          elseif ((vx or vy) > 1) or ((vx or vy) < -1) then
            self.body:applyForce(vx/norm*-60, vy/norm*-60)
          elseif ((vx or vy) > 0.01) or ((vx or vy) < -0.01) then
            self.body:applyForce(vx/norm*-30, vy/norm*-30)
          else
            self.body:setLinearVelocity(0, 0)
          end
          else
            self.body:setLinearVelocity(0, 0)
      end
      local omega = self.body:getAngularVelocity()
      local sign = omega < 0 and -1 or omega > 0 and 1 or 0
      self.body:applyTorque(sign*-100)
    end,

    ballLaunch = function (self)
      if cue then
        local mouseX, mouseY = love.mouse.getPosition()
        mouseX, mouseY = push:toGame(mouseX, mouseY)
        local _x, _y = self.body:getPosition()

        if mouseX and mouseY ~= nil then
          self.vecx = _x+5 - mouseX
          self.vecy = _y+5 - mouseY
          self.vlen = math.sqrt(self.vecx^2+self.vecy^2)
          self.vecx = - (self.vecx / self.vlen)
          self.vecy = - (self.vecy / self.vlen)

          self.vecAngle = math.atan2(mouseY - _y+5, mouseX - _x+5)

          self.body:applyLinearImpulse(self.vecx*5*self.power,self.vecy*5*self.power)
          self.power = 0
          self.step = 0
          self.dir = "up"
        end
      end
    end,

    launchBall = function (self, dt)
      if cue then
        local cuevx, cuevy = self.body:getLinearVelocity()
          function love.keypressed(key)
            if key == "space" and self.step == 0 then
              if cuevx and cuevy < 0.01 then
                self.step = 1
              end
            elseif key == "space" and self.step == 1 then
              if cuevx and cuevy < 0.1 then
                self.step = 0
              self.ballLaunch(self)
            end
          end
        end
            if self.step == 1 then
              
              if self.dir == "up" then
                self.power = math.floor(self.power + (dt * self.rate))
                
                if self.power == 100 then
                  self.dir = "down"
                end
              end
              if  self.dir == "down" then
                self.power = (self.power - (dt * self.rate))
                
                if math.floor(self.power) == 1 then
                  self.dir = "up"
                end
              end
              
            end

    end
  end
  }
end

return Ball