local love = require "love"
local push = require "push"

local Ball = require "Ball"
local Hole = require "Hole"
local Table = require "Table"

love.window.setTitle("Pool") --Title

love.physics.setMeter(10)
world = love.physics.newWorld(0, 0, true)

love.graphics.setDefaultFilter("nearest", "nearest", 1) --disable blurry scaling
love.graphics.setLineStyle("rough")

local gameWidth, gameHeight = 400, 217

local windowWidth, windowHeight = love.window.getDesktopDimensions()
windowWidth, windowHeight = windowWidth*.6, windowHeight*.6

push:setupScreen(gameWidth, gameHeight, windowWidth, windowHeight, {
  fullscreen = false,
  resizable = true,
  pixelperfect = true,
  canvas = true,
  highdpi = true,
})
push:setBorderColor{0, 0, 0} --default value


holes = {}

holes[1] = Hole(195, 12) -- meio cima
holes[2] = Hole(195, 195) -- meio baixo
holes[3] = Hole(12, 193) -- esquerda baixo
holes[4] = Hole(378, 194) -- direita baixo
holes[5] = Hole(378, 13) -- direita cima
holes[6] = Hole(15, 15)

balls = {}

balls[1] = Ball(50, 217/2, true)
balls[2] = Ball(300, 217/2, false) --1
balls[3] = Ball(310, 217/2 + 5, false)--2
balls[4] = Ball(310, 217/2 - 5, false)--2
balls[5] = Ball(320, 217/2, false)--3
balls[6] = Ball(320, 217/2 - 11, false)--3
balls[7] = Ball(320, 217/2 + 11, false)--3
balls[8] = Ball(330, 217/2 - 5, false)--4
balls[9] = Ball(330, 217/2 + 5, false)--4
balls[10] = Ball(330, 217/2 - 16, false)--4
balls[11] = Ball(330, 217/2 + 16, false)--4
table = Table()

callbackDestroy = 0
bodyToDestroy = nil
isCue = false
score = 0

font = love.graphics.getFont()
textScore = love.graphics.newText(font)
textScore:add("score: " .. score)

function ballToHole(a, b, coll)
 _a = a:getUserData()
 _b = b:getUserData()


 if (_a == "Ball" or "Cue Ball") and _b == "Hole" then
  isCue = false
  local body = a:getBody()
  local shape = a:getShape()
  callbackDestroy = 1
  bodyToDestroy = body
  if _a == "Cue Ball" then
    isCue = true
  end
end

end

function destroyBall(body, isCue)
  body:setLinearVelocity(0,0)
  if isCue then
    body:setPosition(50, 217/2)
  else
    score = score + 1
    body:setPosition(-100,-100)
  end

  
end


function love.load()
  world:setCallbacks(ballToHole)

  for i = 1, #balls do
    local ball = balls[i]
    ball:createBall()
  end

  for i = 1, #holes do
    local hole = holes[i]
    hole:create()
  end

  table:create()
end

function love.update(dt)
    world:update(dt)
    print(balls[1].vecAngle)
    if love.keyboard.isDown("f") then
      score = 0
    end
    if callbackDestroy == 1 then
      destroyBall(bodyToDestroy, isCue)
      callbackDestroy = 0
    end

    for i = 1, #balls do
        local ball = balls[i]
        ball:moveBall()
        ball:resetBall()
        ball:launchBall(dt)
      end
end

function love.draw()
  push:apply('start')

  table:draw()
  love.graphics.setColor(1,1,1)
  love.graphics.printf("score: " .. score, 370/2, 100, 200, "left")
  love.graphics.printf("controls: space and mouse direction", 200/2, 120, 500, "left")
  love.graphics.printf("press f to reset", 200/2, 140, 500, "left")

  for i = 1, #balls do
    local ball = balls[i]
    ball:draw()
    ball:drawPowerBar()
    ball:drawPoolCue()
  end


  push:apply('end')
end